# -*- coding: utf-8 -*-
# a chaosdb control class to write events directly into grafana data store as
# annotations using the annotations HTTP API
# The "time" (millisecs since epoch) attribute can be added to each payload 
# to set the annotation time, if missing grafana will consider it's process time for the event.

import requests
import json
from requests.auth import HTTPBasicAuth
import time
import datetime
from logzero import logger
from chaoslib.types import Configuration, Secrets
from .utils import can_connect_to

import logging

# Enabling debugging at http.client level (requests->urllib3->http.client)
# you will see the REQUEST, including HEADERS and DATA, and RESPONSE with HEADERS but without DATA.
# the only thing missing will be the response.body which is not logged.
try: # for Python 3
    from http.client import HTTPConnection
except ImportError:
    from httplib import HTTPConnection
HTTPConnection.debuglevel = 1

logging.basicConfig() # you need to initialize logging, otherwise you will not see anything from requests
logging.getLogger().setLevel(logging.DEBUG)
requests_log = logging.getLogger("urllib3")
requests_log.setLevel(logging.DEBUG)
requests_log.propagate = True

__all__ = [
    "running",
    "cleanup_control",
    "configure_control",
    "before_experiment_control",
    "after_experiment_control",
    "before_method_control",
    "after_method_control",
    "before_activity_control",
    "after_activity_control",
    "post_event"
]

# global defaults
grafana_host = 'localhost'
grafana_port = 3000
grafana_annotation_api_endpoint = '/api/annotations'
grafana_user = 'admin'
grafana_pass = 'admin'
exp_start_time  = int(round(time.time() * 1000))
exp_end_time    = int(round(time.time() * 1000))

def cleanup_control():
    return 1


def configure_control(c: Configuration, s: Secrets):
    global grafana_host
    global grafana_port
    global grafana_annotation_api_endpoint
    global grafana_user
    global grafana_pass

    global exp_start_time
    global exp_end_time
    global dashboardId

    # defaults
    grafana_user = c.get('grafana_user', 'admin')
    grafana_pass = c.get('grafana_pass', 'admin')
    grafana_host = c.get('grafana_host', 'localhost')
    grafana_port = c.get('grafana_port', 3000)
    grafana_annotation_api_endpoint = '/api/annotations'
    exp_start_time  = int(round(time.time() * 1000))
    exp_end_time    = int(round(time.time() * 1000))
    dashboardId = c.get('dashboardId')

    return 1


def running():
    """ Test if the Grafana server is running """

    return can_connect_to(grafana_host, grafana_port)


# Note: annotation region around the experiment is sent
# at experiment end
def before_experiment_control(context: dict, arguments=None):
#     import pdb; pdb.set_trace()
    d = datetime.datetime.now()
    mill = int(d.microsecond/1000)
    exp_start_time = int(round(time.time() * 1000)) + mill
    return 1


def after_experiment_control(context: dict, arguments=None):
    exp_end_time = int(round(time.time() * 1000))

    tags = [ 'chaostoolkit', 'experiment' ]
    text = context['title']

    payload = {
      "dashboardId": dashboardId,
      "time": exp_start_time,
      "isRegion": True,
      "timeEnd": exp_end_time,
      "tags": tags,
      "text": text
    }

    return post_event(payload)


def before_method_control(context: dict, arguments=None):

    tags = [ 'chaostoolkit', 'method', 'before', context['description'] ]
    text = context['title']

    payload = {
      "dashboardId": dashboardId,
      "tags": tags,
      "text": text
    }

    return post_event(payload)

def after_method_control(context: dict, arguments=None):

    tags = [ 'chaostoolkit', 'method', 'after', context['description'] ]
    text = context['title']

    payload = {
      "dashboardId": dashboardId,
      "tags": tags,
      "text": text
    }

    return post_event(payload)

def before_activity_control(context: dict, arguments=None):

    tags = [ 
            'chaostoolkit', 
            'activity', 
            'before', 
            context['type'], 
            context['name'] 
    ]

    import pdb; pdb.set_trace()
    if ( context['provider']['type'] == 'python'):
        tags.append(context['provider']['func'])

    text = f"[{context['type']}]:{context['name']}"

    payload = {
      "dashboardId": dashboardId,
      "tags": tags,
      "text": text
    }

    return post_event(payload)

def after_activity_control(context: dict, arguments=None):

    tags = [ 'chaostoolkit', 'activity', 'after', context['type'], context['name'] ]

    import pdb; pdb.set_trace()
    if ( context['provider']['type'] == 'python'):
        tags.append(context['provider']['func'])

    text = f"[{context['type']}]:{context['name']}"

    payload = {
      "dashboardId": dashboardId,
      "tags": tags,
      "text": text
    }

    return post_event(payload)


def post_event(payload):

    headers = {
        'Accept': 'application/json', 
        'Content-Type': 'application/json'
    }

    r = requests.post("http://{}:{}{}".format(
        grafana_host,
        grafana_port,
        grafana_annotation_api_endpoint),
        auth=HTTPBasicAuth(grafana_user, grafana_pass),
        headers=headers,
        data=json.dumps(payload),
        timeout=0.5)
    
    return r.status_code



import unittest
import sys, os
from chaosdb.influx import encode_payload_in_line_protocol

class TestInfluxPayloadEncoding( unittest.TestCase ):

    def test_one_field_payload(self):

        self.assertEqual(encode_payload_in_line_protocol(
            "measurement", fields={"field1":"value1"}), 
                'measurement field1="value1"')


    def test_two_fields_payload(self):

        self.assertEqual(encode_payload_in_line_protocol(
            "measurement", fields={"field1":"value1","field2":123}), 
                'measurement field1="value1",field2=123')


    def test_fields_and_tags_payload(self):

        self.assertEqual(encode_payload_in_line_protocol(
            "measurement", 
            fields={"field1":"value1","field2":123},
            tags={"tag1":"v1","tag2":"v2"}
        ), 'measurement,tag1=v1,tag2=v2 field1="value1",field2=123')


if __name__ == "__main__":
    unittest.main()



# -*- coding: utf-8 -*-
import sqlite3
import json
import time
from logzero import logger
from chaoslib.types import Configuration, Secrets

__all__ = [
    "cleanup_control",
    "configure_control",
    "before_activity_control",
    "after_activity_control"
]

litedb_filename = ""

def cleanup_control():
    return 1


def configure_control(c: Configuration, s: Secrets):
#     import pdb; pdb.set_trace()
    global litedb_filename
    litedb_filename = c["litedb_filename"]
    return 1


def after_activity_control(context: dict, arguments=None):
    if context['type'] == 'probe':
        return 1

    provider = context['provider']

    return store_action("after", provider)


def before_activity_control(context: dict, arguments=None):
    if context['type'] == 'probe':
        return 1

    provider = context['provider']

    return store_action("before", provider)


def store_action(scope, provider):

#     try:
    logger.debug("store_action")
    logger.debug("Scope: {}".format(scope))
#     import pdb; pdb.set_trace()
    global litedb_filename
    logger.debug("sqlite db file: {}".format(litedb_filename))
    conn = sqlite3.connect(litedb_filename)
#     conn = sqlite3.connect('./trace.db')
    c = conn.cursor()
    stmt = ""
    args = ""
    if 'arguments' in provider:
        args = json.dumps(json.dumps(provider['arguments']))

    if provider['type'] == 'python':
        stmt = "INSERT INTO actions (event_time, scope, type, module, func, args) "\
            "VALUES({},'{}','{}','{}','{}','{}')".format(
                time.time(),
                scope,
                provider['type'],
                provider['module'],
                provider['func'],
                args
            )

    if stmt != "":
        logger.debug("Going to execute cmd: {}".format(stmt))
        c.execute(stmt)
        conn.commit()

    conn.close()
    logger.debug("after_activity_control:end")
#     except:
#         logger.error("Error inserting action")

    return 1



import unittest
import sys, os
from chaosdb.grafana import running, configure_control, before_activity_control, after_activity_control

@unittest.skipIf(running() is False, "Test skipped: Grafana server not running")
class TestControl( unittest.TestCase ):

#     @classmethod
#     def setUpClass(cls):
#         os.system('./scripts/influxd.sh')
#   some initialization...


    def test_before_activity_control(self):

        self.assertEqual(configure_control({}, {} ), 1)

        context =  {
        "type": "action",
        "name": "test action",
        "provider": {
          "type": "python",
          "module": "foo",
          "func": "bar",
          "arguments": {"foo": "bar"}
          }
        }
        self.assertEqual( before_activity_control(context, {}), 200 )


    def test_after_activity_control(self):

        self.assertEqual(configure_control({}, {} ), 1)


        context =  {
        "type": "action",
        "name": "test action",
        "provider": {
          "type": "python",
          "module": "foo",
          "func": "bar"
          }
        }
        self.assertEqual( after_activity_control(context, {}), 200 )


if __name__ == "__main__":
    unittest.main()



import unittest
import sys, os
from chaosdb.litedb import configure_control, before_activity_control, after_activity_control

class TestControl( unittest.TestCase ):

    @classmethod
    def setUpClass(cls):
#         import pdb; pdb.set_trace()
        # trace db creation
        os.system('./scripts/trace.sh')


    def test_before_activity_control(self):

        self.assertEqual(configure_control({
            "litedb_filename": "./trace.db"
        }, {} ), 1)

        context =  {
        "type": "action",
        "name": "test action",
        "provider": {
          "type": "python",
          "module": "foo",
          "func": "bar"
          }
        }
        self.assertEqual( before_activity_control(context, {}), True )


    def test_after_activity_control(self):

        self.assertEqual(configure_control({
            "litedb_filename": "./trace.db"
        }, {} ), 1)

        context =  {
        "type": "action",
        "name": "test action",
        "provider": {
          "type": "python",
          "module": "foo",
          "func": "bar"
          }
        }
        self.assertEqual( after_activity_control(context, {}), True )


if __name__ == "__main__":
    unittest.main()



# -*- coding: utf-8 -*-
import requests
from logzero import logger
from chaoslib.types import Configuration, Secrets
from .utils import can_connect_to

import logging

# Enabling debugging at http.client level (requests->urllib3->http.client)
# you will see the REQUEST, including HEADERS and DATA, and RESPONSE with HEADERS but without DATA.
# the only thing missing will be the response.body which is not logged.
try: # for Python 3
    from http.client import HTTPConnection
except ImportError:
    from httplib import HTTPConnection
HTTPConnection.debuglevel = 1

logging.basicConfig() # you need to initialize logging, otherwise you will not see anything from requests
logging.getLogger().setLevel(logging.DEBUG)
requests_log = logging.getLogger("urllib3")
requests_log.setLevel(logging.DEBUG)
requests_log.propagate = True

__all__ = [
    "running",
    "cleanup_control",
    "configure_control",
    "before_activity_control",
    "after_activity_control",
    "encode_payload_in_line_protocol"
]

# global defaults
influx_host         = "localhost"
influx_port         = 8086
influx_http_endpoint= "/write"
influx_database     = "gatlingdb"

def running():
    """ Test if the InfluxDB server is running """

    return can_connect_to(influx_host, influx_port)


def cleanup_control():
    return 1


def configure_control(c: Configuration, s: Secrets):
    global influx_host
    global influx_port
    global influx_http_endpoint
    global influx_database
    influx_host          = c["influx_host"]
    influx_port          = c["influx_port"]
    influx_http_endpoint = c["influx_http_endpoint"]
    influx_database      = c["influx_database"]
    return 1


def after_activity_control(context: dict, arguments=None):
    if context['type'] == 'probe':
        return 1

    provider = context['provider']

    return store_action("after", provider)


def before_activity_control(context: dict, arguments=None):
    if context['type'] == 'probe':
        return 1

    provider = context['provider']

    return store_action("before", provider)


def store_action(scope, provider):

    # implement InfluxDB line protocol
    # using the requests lib
    # scope is a tag 

#     import pdb; pdb.set_trace()
#     try:
    logger.debug("store_action")
    logger.debug("Scope: {}".format(scope))
#     import pdb; pdb.set_trace()
    global influx_host
    global influx_port
    global influx_http_endpoint
    global influx_database

    logger.debug("Influx host: {}".format(influx_host))
    logger.debug("Influx port: {}".format(influx_port))
    logger.debug("Influx endpoint: {}".format(influx_http_endpoint))
    logger.debug("Influx database: {}".format(influx_database))

    payload = ""

# TODO the measurement should contains the experiment name label
    if provider['type'] == 'python':
        payload = encode_payload_in_line_protocol(
            "chaos_toolkit.actions",
            tags={
                "experiment":"exp1", 
                "scope":scope
            },
            fields=provider
        )

    r = requests.post("http://{}:{}{}".format(
        influx_host, influx_port, influx_http_endpoint),
        params = {"db":influx_database},
        data = payload)

    if r.status_code != 204:
        logger.error("Error sending data to InfluxDB: {}".format(r.json()))

    logger.debug("store_action ended")
#     except:
#         logger.error("Error inserting action")

    return 1

# Encodes payload
# Influx will automatically add the timestamp...
def encode_payload_in_line_protocol(measurement, fields: dict, tags={}):

    payload = measurement

    # never use quotes for tags...
    for k in sorted(tags):
        payload = "{},{}={}".format(payload, k, tags[k])

    payload = payload + " "

#     import pdb; pdb.set_trace()
    for k,v in fields.items():
        if isinstance(v, str):
            payload = "{}{}=\"{}\",".format(payload, k, v)
        elif isinstance(v, dict):
            # TODO : we have to encode dicts properly...
            continue
        else:
            payload = "{}{}={},".format(payload, k, v)

    payload = payload.rstrip(',')
    return payload


